/*********************************************
 * BIOE-Library - device block for scicos
 * Copyright (C) 2010  Weichinger Klaus, snaky.1@gmx.at
 * 
 * License: CC-GNU GPL
 * http://creativecommons.org/licenses/GPL/2.0/ 
 *********************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *********************************************/

#include "scicosheaders.h"
#include "bioe.h"
// read=BIOE_Transaction(basePort,channel,Device,Endpoint,Value,speedfactor,pausefactor)
// BIOE_ConfigTransaction(basePort,channel,Device,Action,Data,speedfactor,pausefactor)

static void init(scicos_block *block)
{
  int port,cs,speedfaktor,pausefaktor,adr,dtn;
  int s0,s1,s2,s3,s4,s5,s6,s7;
  int ep2,ep3,ep4,ep5,ep6,ep7,ep8,ep9,ep10,ep11,ep12,ep13,ep14;
  int settings,endpoints;
  int res;
  char cmd[30];

  *block->work=scicos_malloc(sizeof(int)*4);
  int *params=*block->work;
  params[0]=BIOE_ReadConfigFile_Int("bioesettings.conf","bioe_port",block->ipar[0]);
  params[1]=BIOE_ReadConfigFile_Int("bioesettings.conf","bioe_cs",block->ipar[1]);
  params[2]=BIOE_ReadConfigFile_Int("bioesettings.conf","bioe_speedfactor",block->ipar[2]);
  params[3]=BIOE_ReadConfigFile_Int("bioesettings.conf","bioe_pausefactor",block->ipar[3]);
  
  sprintf(cmd,"bioe_port%d",block->ipar[4]);
  params[0]=BIOE_ReadConfigFile_Int("bioesettings.conf",cmd,params[0]);
  sprintf(cmd,"bioe_cs%d",block->ipar[4]);
  params[1]=BIOE_ReadConfigFile_Int("bioesettings.conf",cmd,params[1]);
  sprintf(cmd,"bioe_speedfactor%d",block->ipar[4]);
  params[2]=BIOE_ReadConfigFile_Int("bioesettings.conf",cmd,params[2]);
  sprintf(cmd,"bioe_pausefactor%d",block->ipar[4]);
  params[3]=BIOE_ReadConfigFile_Int("bioesettings.conf",cmd,params[3]);  

  port=params[0];
  cs=params[1];
  speedfaktor=params[2];
  pausefaktor=params[3];

  sprintf(cmd,"%d",block->ipar[4]);
  BIOE_WriteConfigFile("bioesettings_default.conf",cmd,port,cs,speedfaktor,pausefaktor);

  adr=block->ipar[5];
  dtn=block->ipar[6];
  s0=block->ipar[7];
  s1=block->ipar[8];
  s2=block->ipar[9];
  s3=block->ipar[10];
  s4=block->ipar[11];
  s5=block->ipar[12];
  s6=block->ipar[13];
  s7=block->ipar[14];

  ep2=block->ipar[15];
  ep3=block->ipar[16];
  ep4=block->ipar[17];
  ep5=block->ipar[18];
  ep6=block->ipar[19];
  ep7=block->ipar[20];
  ep8=block->ipar[21];
  ep9=block->ipar[22];
  ep10=block->ipar[23];
  ep11=block->ipar[24];
  ep12=block->ipar[25];
  ep13=block->ipar[26];
  ep14=block->ipar[27];

  printf("BIOE-IO-Init: Port %d, CS%d, Device %d\n",port,cs,adr);
  BIOE_Init(port); // LPT initialisieren
  if (dtn>0) {
    res=BIOE_ConfigTransaction(port,cs,adr,0,dtn,speedfaktor,pausefaktor); // Gerät initialisieren
    if (res==-1) {
      printf("              DTN %d ...Error\n",dtn);
    } else {
      printf("              DTN %d\n",dtn);
    }
  }
  res=0;
  endpoints=0;
  if(ep2>=0) {
    res+=BIOE_Transaction(port,cs,adr,2,ep2,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep3>=0) {
    res+=BIOE_Transaction(port,cs,adr,3,ep3,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep4>=0) {
    res+=BIOE_Transaction(port,cs,adr,4,ep4,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep5>=0) {
    res+=BIOE_Transaction(port,cs,adr,5,ep5,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep6>=0) {
    res+=BIOE_Transaction(port,cs,adr,6,ep6,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep7>=0) {
    res+=BIOE_Transaction(port,cs,adr,7,ep7,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep8>=0) {
    res+=BIOE_Transaction(port,cs,adr,8,ep8,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep9>=0) {
    res+=BIOE_Transaction(port,cs,adr,9,ep9,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep10>=0) {
    res+=BIOE_Transaction(port,cs,adr,10,ep10,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep11>=0) {
    res+=BIOE_Transaction(port,cs,adr,11,ep11,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep12>=0) {
    res+=BIOE_Transaction(port,cs,adr,12,ep12,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep13>=0) {
    res+=BIOE_Transaction(port,cs,adr,13,ep13,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep14>=0) {
    res+=BIOE_Transaction(port,cs,adr,14,ep14,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if (endpoints==1) {
    if (res==0) {
      printf("              Default Endpointvalues initialized.\n");
    } else {
      printf("              Default Endpointvalues initializing ... Error\n");
    }
  }
  if (dtn>0) {
    settings=0;
    res=0;
    if (s0>=0) {
      res+=BIOE_ConfigTransaction(port,cs,adr,0x10,s0,speedfaktor,pausefaktor)<0?-1:0;
      settings=1;
    }
    if (s1>=0) {
      res+=BIOE_ConfigTransaction(port,cs,adr,0x11,s1,speedfaktor,pausefaktor)<0?-1:0;
      settings=1;
    }
    if (s2>=0) {
      res+=BIOE_ConfigTransaction(port,cs,adr,0x12,s2,speedfaktor,pausefaktor)<0?-1:0;
      settings=1;
    }
    if (s3>=0) {
      res+=BIOE_ConfigTransaction(port,cs,adr,0x13,s3,speedfaktor,pausefaktor)<0?-1:0;
      settings=1;
    }
    if (s4>=0) {
      res+=BIOE_ConfigTransaction(port,cs,adr,0x14,s4,speedfaktor,pausefaktor)<0?-1:0;
      settings=1;
    }
    if (s5>=0) {
      res+=BIOE_ConfigTransaction(port,cs,adr,0x15,s5,speedfaktor,pausefaktor)<0?-1:0;
      settings=1;
    }
    if (s6>=0) {
      res+=BIOE_ConfigTransaction(port,cs,adr,0x16,s6,speedfaktor,pausefaktor)<0?-1:0;
      settings=1;
    }
    if (s7>=0) {
      res+=BIOE_ConfigTransaction(port,cs,adr,0x17,s7,speedfaktor,pausefaktor)<0?-1:0;
      settings=1;
    }
    if (settings>0) {
      res+=BIOE_ConfigTransaction(port,cs,adr,0x02,1,speedfaktor,pausefaktor)<0?-1:0; // Settings Done senden
    }
    if (settings==1) {
      if (res==0) {
        printf("              Settings initialized.\n");
      } else {
        printf("              Settings initializing ... Error\n");
      }
    }
  }
}

static void in(scicos_block *block)
{
}

static void out(scicos_block *block)
{
}

static void end(scicos_block *block)
{
  int port,cs,speedfaktor,pausefaktor,adr,dtn;
  int s0,s1,s2,s3,s4,s5,s6,s7;
  int ep2,ep3,ep4,ep5,ep6,ep7,ep8,ep9,ep10,ep11,ep12,ep13,ep14;
  int endpoints;
  int res;

  int *params=*block->work;
  port=params[0];
  cs=params[1];
  speedfaktor=params[2];
  pausefaktor=params[3];

  adr=block->ipar[4];
  dtn=block->ipar[5];
  s0=block->ipar[6];
  s1=block->ipar[7];
  s2=block->ipar[8];
  s3=block->ipar[9];
  s4=block->ipar[10];
  s5=block->ipar[11];
  s6=block->ipar[12];
  s7=block->ipar[13];

  ep2=block->ipar[14];
  ep3=block->ipar[15];
  ep4=block->ipar[16];
  ep5=block->ipar[17];
  ep6=block->ipar[18];
  ep7=block->ipar[19];
  ep8=block->ipar[20];
  ep9=block->ipar[21];
  ep10=block->ipar[22];
  ep11=block->ipar[23];
  ep12=block->ipar[24];
  ep13=block->ipar[25];
  ep14=block->ipar[26];
  BIOE_Close(port); // LPT freigeben
  BIOE_Init(port); // LPT initialisieren


  printf("BIOE-IO-End: Port %d, CS%d, Device %d\n",port,cs,adr);
  res=0;
  endpoints=0;
  if(ep2>=0) {
    res+=BIOE_Transaction(port,cs,adr,2,ep2,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep3>=0) {
    res+=BIOE_Transaction(port,cs,adr,3,ep3,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep4>=0) {
    res+=BIOE_Transaction(port,cs,adr,4,ep4,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep5>=0) {
    res+=BIOE_Transaction(port,cs,adr,5,ep5,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep6>=0) {
    res+=BIOE_Transaction(port,cs,adr,6,ep6,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep7>=0) {
    res+=BIOE_Transaction(port,cs,adr,7,ep7,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep8>=0) {
    res+=BIOE_Transaction(port,cs,adr,8,ep8,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep9>=0) {
    res+=BIOE_Transaction(port,cs,adr,9,ep9,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep10>=0) {
    res+=BIOE_Transaction(port,cs,adr,10,ep10,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep11>=0) {
    res+=BIOE_Transaction(port,cs,adr,11,ep11,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep12>=0) {
    res+=BIOE_Transaction(port,cs,adr,12,ep12,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep13>=0) {
    res+=BIOE_Transaction(port,cs,adr,13,ep13,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if(ep14>=0) {
    res+=BIOE_Transaction(port,cs,adr,14,ep14,speedfaktor,pausefaktor);
    endpoints=1;
  }
  if (endpoints==1) {
    if (res==0) {
      printf("              Default Endpointvalues restored.\n");
    } else {
      printf("              Default Endpointvalues restoring ... Error\n");
    }
  }
  BIOE_Close(port); // LPT freigeben
  scicos_free(*block->work);
}



void bioe_dev_fnc(scicos_block *block,int flag)
{
  if (flag==1){          /* set output */
    out(block);
  }
  if (flag==2){          /* get input */
    in(block);
  }
  else if (flag==5){     /* termination */ 
    end(block);
  }
  else if (flag ==4){    /* initialisation */
    init(block);
  }
}


