// *********************************************************************
// BIOE-Library - device block interface function for scicos
// Copyright (C) 2010  Weichinger Klaus, snaky.1@gmx.at
// 
// License: CC-GNU GPL
// http://creativecommons.org/licenses/GPL/2.0/ 
// *********************************************************************
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor,
// Boston, MA  02110-1301, USA.
// *********************************************************************
 
function [x, y, typ] = bioe_dev(job, arg1, arg2)
x = []; y = []; typ = [];
select job
case 'plot' then
  graphics=arg1.graphics;exprs=graphics.exprs
  icon_line1=exprs(1);
  icon_line2='ADR='+exprs(2)+' DTN='+exprs(3);
  icon_line3=exprs(6);
  standard_draw(arg1)
case 'getinputs' then
  [x, y, typ] = standard_inputs(arg1)
case 'getoutputs' then
  [x, y, typ] = standard_outputs(arg1)
case 'getorigin' then
  [x, y] = standard_origin(arg1)
case 'set' then
  x = arg1;
  model = arg1.model; graphics = arg1.graphics;
  exprs = graphics.exprs;
  parametercaptionsorig=['Caption','Deviceaddress (0..15)','Devicetypenumber DTN',..
                'Settings','Default OUT-EP-Values','Interface [PORT,CS,SPEED,PAUSE,ID]'];
  parametercaptions=parametercaptionsorig
  while %t do
    [ok,caption,adr,dtn,settings,defaultep,interface,exprs] =..	// acquire parameters in dialog box
        getvalue('Set BIOE-Device Parameters (by Weichinger Klaus, snaky.1@gmx.at)',..
                 parametercaptions,..
                 list('str',1,'vec',1,'vec',1,'mat',[1,-1],'mat',[1,-1],'mat',[1,5]),..
                 exprs)

    parametercaptions=parametercaptionsorig
    if ~ok then
      break
    end

    s0=-1;s1=-1;s2=-1;s3=-1;s4=-1;s5=-1;s6=-1;s7=-1;

    ep2=-1;ep3=-1;ep4=-1;ep5=-1;ep6=-1;
    ep7=-1;ep8=-1;ep9=-1;ep10=-1;ep11=-1;
    ep12=-1;ep13=-1;ep14=-1;

    // berprfung der eingegebenen Parameter
    if (sum([0:15]==adr)==0) then
      ok=%f
      parametercaptions(2)='*'+parametercaptions(2)
    end
    if (sum([-1:255]==dtn)==0) then
      ok=%f
      parametercaptions(3)='*'+parametercaptions(3)
    end
    if (modulo(max(size(settings)),2)==1) then
      ok=%f
      parametercaptions(4)='*'+parametercaptions(4)
    end
    if (modulo(max(size(defaultep)),2)==1) then
      ok=%f
      parametercaptions(5)='*'+parametercaptions(5)
    end
    if (sum([1:3]==interface(2))==0) | (interface(3)<0) | (interface(4)<0) then
      ok=%f
      parametercaptions(6)='*'+parametercaptions(6)
    end

    // Analyse und Zuweisung der Settings
    if ok then
      len=max(size(settings))
      for i=[1:2:len]
        n=settings(1,i)
        v=settings(1,i+1)
        found=0
        if (sum([0:7]==n)==0) | (v<0) then
          ok=%f
          parametercaptions(4)='*'+parametercaptions(4)
        else
          if (n==0) then, s0=v, end
          if (n==1) then, s1=v, end
          if (n==2) then, s2=v, end
          if (n==3) then, s3=v, end
          if (n==4) then, s4=v, end
          if (n==5) then, s5=v, end
          if (n==6) then, s6=v, end
          if (n==7) then, s7=v, end
        end
      end
    end

    // Analyse und Zuweisung der Default-Endpoint Values
    if ok then
      len=max(size(defaultep))
      for i=[1:2:len]
        n=defaultep(i)
        v=defaultep(i+1)
        if (sum([2:14]==n)==0) | (v<0) then
          ok=%f
          parametercaptions(5)='*'+parametercaptions(5)
        else
          if (n==2) then, ep2=v, end
          if (n==3) then, ep3=v, end
          if (n==4) then, ep4=v, end
          if (n==5) then, ep5=v, end
          if (n==6) then, ep6=v, end
          if (n==7) then, ep7=v, end
          if (n==8) then, ep8=v, end
          if (n==9) then, ep9=v, end
          if (n==10) then, ep10=v, end
          if (n==11) then, ep11=v, end
          if (n==12) then, ep12=v, end
          if (n==13) then, ep13=v, end
          if (n==14) then, ep14=v, end
        end
      end
    end


    if ok then
      [model, graphics, ok] = check_io(model, graphics, [], [], [], [])
    end


    ipar=[interface(1),interface(2),interface(3),interface(4),interface(5),..
          adr,dtn,s0,s1,s2,s3,s4,s5,s6,s7,..
          ep2,ep3,ep4,ep5,ep6,ep7,ep8,ep9,ep10,ep11,ep12,ep13,ep14];
    ipar=int(ipar(:));

    if ok then
      graphics.exprs = exprs;
      model.rpar = []			// rpar can be passed on to xrtailab
      model.ipar = ipar;
      model.dstate = [1];
      x.graphics = graphics;
      x.model = model
      break
    end

    x_message(['Some BIOE-Device Parameters are not valid';'The Parameters are marked with a *.']);

  end
case 'define' then			// initialize parameters

  caption='IO-Device'
  adr=0
  dtn=0
  settings=[]
  defaultep=[]
  interface=[888,1,0,9,0]

  s0=-1;s1=-1;s2=-1;s3=-1;s4=-1;s5=-1;s6=-1;s7=-1;

  ep2=-1;ep3=-1;ep4=-1;ep5=-1;ep6=-1
  ep7=-1;ep8=-1;ep9=-1;ep10=-1;ep11=-1
  ep12=-1;ep13=-1;ep14=-1 

  ipar=[interface(1),interface(2),interface(3),interface(4),interface(5),..
        adr,dtn,s0,s1,s2,s3,s4,s5,s6,s7,..
        ep2,ep3,ep4,ep5,ep6,ep7,ep8,ep9,ep10,ep11,ep12,ep13,ep14];
  ipar=int(ipar(:));


  model = scicos_model()
  model.sim = list('bioe_dev_fnc', 4)

  model.in = []
  model.out = []

  model.evtin = []
  model.rpar=[]
  model.ipar = ipar;

  model.dstate=[1];
  model.blocktype='d'
  model.dep_ut=[%t %f]
  exprs=[caption,sci2exp(adr),sci2exp(dtn),sci2exp(settings),sci2exp(defaultep),sci2exp(interface)]
  gr_i=['xstringb(orig(1),orig(2),[icon_line1;icon_line2;icon_line3],sz(1),sz(2),''fill'')']

  x = standard_define([5 3], model, exprs, gr_i)
end
endfunction
