/*********************************************
 * BIOE-Library - endpoint block for scicos
 * Copyright (C) 2010  Weichinger Klaus, snaky.1@gmx.at
 * 
 * License: CC-GNU GPL
 * http://creativecommons.org/licenses/GPL/2.0/ 
 *********************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *********************************************/
 
#include "scicosheaders.h"
#include "bioe.h"

static void init(scicos_block *block)
{
  int port,cs,speedfaktor,pausefaktor;
  char cmd[30];

  *block->work=scicos_malloc(sizeof(int)*4);
  int *params=*block->work;
  params[0]=BIOE_ReadConfigFile_Int("bioesettings.conf","bioe_port",block->ipar[0]);
  params[1]=BIOE_ReadConfigFile_Int("bioesettings.conf","bioe_cs",block->ipar[1]);
  params[2]=BIOE_ReadConfigFile_Int("bioesettings.conf","bioe_speedfactor",block->ipar[2]);
  params[3]=BIOE_ReadConfigFile_Int("bioesettings.conf","bioe_pausefactor",block->ipar[3]);
  
  sprintf(cmd,"bioe_port%d",block->ipar[4]);
  params[0]=BIOE_ReadConfigFile_Int("bioesettings.conf",cmd,params[0]);
  sprintf(cmd,"bioe_cs_%d",block->ipar[4]);
  params[1]=BIOE_ReadConfigFile_Int("bioesettings.conf",cmd,params[1]);
  sprintf(cmd,"bioe_speedfactor_%d",block->ipar[4]);
  params[2]=BIOE_ReadConfigFile_Int("bioesettings.conf",cmd,params[2]);
  sprintf(cmd,"bioe_pausefactor_%d",block->ipar[4]);
  params[3]=BIOE_ReadConfigFile_Int("bioesettings.conf",cmd,params[3]);  

  port=params[0];
  cs=params[1];
  speedfaktor=params[2];
  pausefaktor=params[3];

  sprintf(cmd,"%d",block->ipar[4]);
  BIOE_WriteConfigFile("bioesettings_default.conf",cmd,port,cs,speedfaktor,pausefaktor);
}

static void in(scicos_block *block)
{
  double i;
  int port,cs,adr,ep,speedfaktor,pausefaktor,io,x;
  int res;

  int *params=*block->work;
  port=params[0];
  cs=params[1];
  speedfaktor=params[2];
  pausefaktor=params[3];

  adr=block->ipar[5];
  ep=block->ipar[6];
  io=block->ipar[7];


  if (io==1) {
    double *u=block->inptr[0];
    i=u[0]+0.5;
    if (i<0) i=0;
    if (i>65535) i=65535;
    x=(unsigned short) i;
    res=BIOE_Transaction(port,cs,adr,ep,x,speedfaktor,pausefaktor);
    if (res==-1) {
      printf("BIOE-IO-Error: Device %d, EP %d\n",adr,ep);
    }
  }
}

static void out(scicos_block *block)
{
  int port,cs,adr,ep,speedfaktor,pausefaktor,io,x;
  int *params=*block->work;
  port=params[0];
  cs=params[1];
  speedfaktor=params[2];
  pausefaktor=params[3];

  adr=block->ipar[5];
  ep=block->ipar[6];
  io=block->ipar[7];

  if (io==0) {
    x=BIOE_Transaction(port,cs,adr,ep,0,speedfaktor,pausefaktor);
    if (x==-1) {
      printf("BIOE-IO-Error: Device %d, EP %d\n",adr,ep);
    } else {
      double *y=block->outptr[0];
      y[0]=x;
    }
  }
}

static void end(scicos_block *block)
{
  scicos_free(*block->work);
}



void bioe_ep_fnc(scicos_block *block,int flag)
{
  if (flag==1){          /* set output */
    out(block);
  }
  if (flag==2){          /* get input */
    in(block);
  }
  else if (flag==5){     /* termination */ 
    end(block);
  }
  else if (flag ==4){    /* initialisation */
    init(block);
  }
}


