// *********************************************************************
// BIOE-Library - endpoint block interface function for scicos
// Copyright (C) 2010  Weichinger Klaus, snaky.1@gmx.at
// 
// License: CC-GNU GPL
// http://creativecommons.org/licenses/GPL/2.0/ 
// *********************************************************************
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor,
// Boston, MA  02110-1301, USA.
// *********************************************************************

function [x, y, typ] = bioe_ep(job, arg1, arg2)
x = []; y = []; typ = [];
select job
case 'plot' then
  graphics=arg1.graphics;exprs=graphics.exprs
  icon_line1=exprs(1)
  icon_line2='ADR='+exprs(2)+' EP'+exprs(3)
  icon_line3=exprs(5)
  standard_draw(arg1)
case 'getinputs' then
  [x, y, typ] = standard_inputs(arg1)
case 'getoutputs' then
  [x, y, typ] = standard_outputs(arg1)
case 'getorigin' then
  [x, y] = standard_origin(arg1)
case 'set' then
  x = arg1;
  model = arg1.model; graphics = arg1.graphics;
  exprs = graphics.exprs;
  parametercaptionsorg=..
    ['Caption','Deviceaddress (0..15)','Endpoint EP (2..14)',..
    'In=0, Out=1','Interface [PORT,CS,SPEED,PAUSE]']
  parametercaptions=parametercaptionsorg
  while %t do
    [ok,caption,adr,ep,io,interface,exprs] =..	// acquire parameters in dialog box
        getvalue('Set BIOE-Endpoint Parameters (by Weichinger Klaus, snaky.1@gmx.at)',..
                 parametercaptions,..
                 list('str',1,'vec',1,'vec',1,'vec',1,'vec',5),..
                 exprs)
    parametercaptions=parametercaptionsorg
    if ~ok then break, end

    // Überprüfung der eingegebenen Parameter
    if (sum([0:15]==adr)==0) then
      ok=%f
      parametercaptions(2)='*'+parametercaptions(2)
    end
    if (sum([2:14]==ep)==0) then
      ok=%f
      parametercaptions(3)='*'+parametercaptions(3)
    end
    if (sum([0:1]==io)==0) then
      ok=%f
      parametercaptions(4)='*'+parametercaptions(4)
    end
    if (sum([1:3]==interface(2))==0) | (interface(3)<0) | (interface(4)<0) then
      ok=%f
      parametercaptions(5)='*'+parametercaptions(5)
    end

    if ok then
      if (io==0) then
        in = []
        out = [1]
      end
      if (io==1) then
        in = [1]
        out = []
      end

      [model, graphics, ok] = check_io(model, graphics, in, out, 1, [])
    end
    ipar=[interface(1),interface(2),interface(3),interface(4),interface(5),adr,ep,io];
    ipar=int(ipar(:));

    if ok then
      graphics.exprs = exprs;
      model.rpar = []			// rpar can be passed on to xrtailab
      model.ipar = ipar;
      model.dstate = [1];
      x.graphics = graphics;
      x.model = model
      break
    end
    x_message(['Some BIOE-Endpoint Parameters are not valid';'The Parameters are marked with a *.']);
  end
case 'define' then			// initialize parameters
  caption='IO-Endpoint'
  adr=0
  ep = 2
  io=0
  interface=[888,1,0,9,0];

  model = scicos_model()
  model.sim = list('bioe_ep_fnc', 4)

  if (io==0) then
    model.in = []
    model.out = [1]
  end
  if (io==1) then
    model.in = [1]
    model.out = []
  end

  ipar=[interface(1),interface(2),interface(3),interface(4),interface(5),adr,ep,io];
  ipar=int(ipar(:));

  model.evtin = 1
  model.rpar=[]
  model.ipar=ipar;
  model.dstate=[1];
  model.blocktype='d'
  model.dep_ut=[%t %f]
  exprs=[caption,sci2exp(adr),sci2exp(ep),sci2exp(io),sci2exp(interface)]
  gr_i=['xstringb(orig(1),orig(2),[icon_line1;icon_line2;icon_line3],sz(1),sz(2),''fill'')']

  x = standard_define([5 3], model, exprs, gr_i)
end
endfunction
